/*
    Proteus RPG Engine - A flexible 3D story based RPG engine.
    Copyright (C) 2002  Sam Yatchmenoff

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __bsp_h_
#define __bsp_h_

#include <SDL.h>
#include "types.h"

struct bsp_lump {
  Uint32    offset;     // offset (in bytes) of the data from the beginning of the file
  Uint32    length;     // length (in bytes) of the data
};

struct bsp_header {
  Uint32    magic;      // magic number ("IBSP")
  Uint32    version;    // version of the BSP format (38)
  
  bsp_lump  lump[19];   // directory of the lumps
};


struct bsp_face {
  Uint16   plane;             // index of the plane the face is parallel to
  Uint16   plane_side;        // set if the normal is parallel to the plane normal
  
  Uint32   first_edge;        // index of the first edge (in the face edge array)
  Uint16   num_edges;         // number of consecutive edges (in the face edge array)
  
  Uint16   texture_info;      // index of the texture info structure	
  
  Uint8    lightmap_syles[4]; // styles (bit flags) for the lightmaps
  Uint32   lightmap_offset;   // offset of the lightmap (in bytes) in the lightmap lump
};

struct bsp_plane {
  vertex3f_t   normal;      // A, B, C components of the plane equation
  float     distance;    // D component of the plane equation
  Uint32    type;        // ?  
};

struct bsp_node {
  Uint32   plane;             // index of the splitting plane (in the plane array)
  
  Sint32   front_child;       // index of the front child node or leaf
  Sint32   back_child;        // index of the back child node or leaf
  
  vertex3s_t  bbox_min;          // minimum x, y and z of the bounding box
  vertex3s_t  bbox_max;          // maximum x, y and z of the bounding box
  
  Uint16   first_face;        // index of the first face (in the face array)
  Uint16   num_faces;         // number of consecutive edges (in the face array)
};


struct bsp_leaf {
  Uint32   brush_or;          // ?
  
  Uint16   cluster;           // -1 for cluster indicates no visibility information
  Uint16   area;              // ?
 
  vertex3s_t  bbox_min;          // bounding box minimums
  vertex3s_t  bbox_max;          // bounding box maximums
  
  Uint16   first_leaf_face;   // index of the first face (in the face leaf array)
  Uint16   num_leaf_faces;    // number of consecutive edges (in the face leaf array)
  
  Uint16   first_leaf_brush;  // ?
  Uint16   num_leaf_brushes;  // ?
};

struct bsp_texinfo {
  vertex3f_t  u_axis;
  float    u_offset;
  
  vertex3f_t  v_axis;
  float    v_offset;
  
  Uint32   flags;
  Uint32   value;
  
  char     texture_name[32];
  
  Uint32   next_texinfo;
};


struct bsp_vis_offset {
  Uint32 pvs;   // offset (in bytes) from the beginning of the visibility lump
  Uint32 phs;   // ?
};

void *load_map_bsp(char *filename);    // returns a Map ready to be used
void free_map_bsp(void *data);

#endif




