/*
    Proteus RPG Engine - A flexible 3D story based RPG engine.
    Copyright (C) 2002  Sam Yatchmenoff

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __character_h
#define __character_h

#include <string>
#include "types.h"
#include "model.h"
#include "resman.h"
#include <utility>
#include "bitmap.h"

struct CharDescription {
  string name;
  int age;
};

struct CharAttributes {
  int attack;
  int defence;
  int magic_attack;
  int magic_defence;
  int speed;
  int constitution;
};

struct CharStatus {
  int HP;
  int MP;
};

class Character {
public:
  int load(const string &);  
  Resource<Bitmap> get_portrait_img(void);
  Resource<Model> get_model(void);
  void set_location(float x,  float y, float z);
  
  CharDescription *get_description() { return &description; };
  CharAttributes *get_attributes() { return &attributes; };
  CharStatus *get_status() { return &status; };
 
 private:
  Resource<Model> model;
  Resource<Bitmap> portrait_img;
  
  CharDescription description;
  CharAttributes attributes;
  CharStatus status;
};

#endif
