/*
    Proteus RPG Engine - A flexible 3D story based RPG engine.
    Copyright (C) 2002  Sam Yatchmenoff

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <windows.h>

#include <iostream>
#include <GL/gl.h>
#include "universe.h"
#include "input.h"
#include "character.h"
#include "visuals.h"
#include "video.h"
#include "bitmap.h"
#include "text.h"

using namespace std;

Bitmap *bitmap; // XXX

void Universe::menu_init() {
	bitmap = new Bitmap("test.png", 0, 0);
	section = Stats;
	cur_item = NULL;
	cout << "menu_init()\n";
}

void Universe::menu_shutdown() {
}

void Universe::menu_update() {
	if(input_get_key(I_KEY_ESC) == true) {
		input_set_key(I_KEY_ESC, false);
		adventure_start();
	}
	switch(section) {
	case Stats:
		menu_update_stats();
		break;
	case Inventory:
		menu_update_inv();
		break;
	}
}

void Universe::menu_update_stats() {
	if(input_get_key(I_KEY_I) == true) {
		input_set_key(I_KEY_I, false);
		section = Inventory;
	}
	menu_draw_stats();
}

void Universe::menu_update_inv() {
	if(cur_item == NULL && party.get_inv().size() > 0) {
		cur_item = party.get_inv().begin();
	}
	
	if(input_get_key(I_KEY_S) == true) {
      input_set_key(I_KEY_S, false);
      section = Stats;
   }
	if(input_get_key(I_KEY_SPACE) == true) {
		input_set_key(I_KEY_SPACE, false);
		(*cur_item)->use_on(party.get_character(0));
		party.get_inv().erase(cur_item);
		cur_item = party.get_inv().begin();
	}
	if(input_get_key(I_KEY_DOWN) == true) {
		input_set_key(I_KEY_DOWN, false);
		if(cur_item != --(party.get_inv().end()))
			cur_item++;
	}
	if(input_get_key(I_KEY_UP) == true) {
		input_set_key(I_KEY_UP, false);
		if(cur_item != party.get_inv().begin())
			cur_item--;
	}  

	menu_draw_inv();
}

void Universe::menu_start() {
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-1.0f, 1.0f, -1.0f, 1.0f, 1.0f, -1.0f);
	glMatrixMode(GL_MODELVIEW);
	set_game_state(GS_MENU);

}

void Universe::menu_draw_inv() {
	list<Resource<Object> >::iterator invi;
	int y = 40;
	for(invi = party.get_inv().begin(); invi != party.get_inv().end(); invi++) {
		if(invi == cur_item) {
//			add_visual(new Text(40, y, 1.0, 1.0, 1.0,
//							"A %s", ((*invi)->get_type()).c_str()));
		} else {
//		   add_visual(new Text(40, y, 0.5, 0.5, 0.5,
//							"A %s", ((*invi)->get_type()).c_str()));
		}
		y += 30;
   }
}

void Universe::menu_draw_stats() {
	// get and output character stats
	Character *character = party.get_character(0); 
	CharDescription *description = character->get_description();
	CharStatus *status = character->get_status();
	CharAttributes *attributes = character->get_attributes();

	add_visual(new Text(270, 10, 1.0f, 1.0f, 1.0f,
				  "Name: %s", description->name.c_str()));
	add_visual(new Text(270, 40, 1.0f, 1.0f, 1.0f,
				  "Age: %d", description->age));
	add_visual(new Text(270, 70, 1.0f, 1.0f, 1.0f,
				  "HP: %d", status->HP));
	add_visual(new Text(270, 100, 1.0f, 1.0f, 1.0f,
				  "MP: %d", status->MP));
	add_visual(new Text(270, 130, 1.0f, 1.0f, 1.0f,
				  "Attack: %d", attributes->attack));
	add_visual(new Text(270, 160, 1.0f, 1.0f, 1.0f,
				  "Defence: %d", attributes->defence));
	add_visual(new Text(270, 190, 1.0f, 1.0f, 1.0f,
				  "Magic Attack: %d", attributes->magic_attack));
	add_visual(new Text(270, 220, 1.0f, 1.0f, 1.0f,
				  "Magic Defence: %d", attributes->magic_defence));
	add_visual(new Text(270, 250, 1.0f, 1.0f, 1.0f,
				  "Speed: %d", attributes->speed));
 
	add_visual(bitmap);
}
