/*
    Proteus RPG Engine - A flexible 3D story based RPG engine.
    Copyright (C) 2002  Sam Yatchmenoff

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <windows.h>

#include <SDL/SDL.h>
#include <iostream>
#include <stdlib.h>
#include "universe.h"
#include "video.h"
#include "resman.h"
#include "object.h"
#include "text.h"

void clean_up(void);

Universe *universe;
GameOptions options;

int main(int argc, char *argv[]) {  
	if(SDL_Init(SDL_INIT_VIDEO) != 0) {
		cerr << "Unable to initialize SDL library: " << SDL_GetError() << endl;
		exit(1);
	}
	atexit(SDL_Quit);

	if(init_video() == -1) {
		std::cerr << "Unable to initialize video subsystem, exiting\n";
		exit(1);
	}
	if(init_resman() == -1) {
		std::cerr << "Unable to initialize resource manager subsystem\n";
		exit(1);
	}
  
 	// Load up some test fonts
	load_fonts();

	
	if(argc > 1) {
		options.module_name = argv[1];
	} else {
 		options.module_name = "modules/BASE-voyage";
	}
	Universe *universe = new Universe(&options);
	if(universe->error()) {
		std::cerr << "Error loading module, quiting (potentially) cleanly\n";
		clean_up();
	}
	universe->big_bang();
  
	clean_up();
	return 0;
}

void clean_up() {
	/*
	shutdown_video();
	shutdown_sound();
	shutdown_resman(); 
	delete universe;
	*/
	exit(0);
}

string get_module_name() {
	return options.module_name;
}

