/*
    Proteus RPG Engine - A flexible 3D story based RPG engine.
    Copyright (C) 2002  Sam Yatchmenoff

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <windows.h>

#include <iostream>
#include <GL/gl.h>
#include <GL/glu.h>
#include "map.h"
#include "bsp.h"
#include "video.h"
#include "party.h"
#include "universe.h"
#include <SDL_image.h>
#include "texture.h"

int Map::load(const string &filename) { // thanx id Software =]
  FILE *fd;
  char *buf;
  int buf_size, buf_pos;
  
  // Switch over to C++ io?
  if((fd = fopen(filename.c_str(), "rb")) == NULL) {
    cout << "Unable to open file: " << filename << endl;
  }
  
  fseek(fd, 0, SEEK_END);
  buf_size = ftell(fd);
  fseek(fd, 0, SEEK_SET);
  buf = new char[buf_size];           // alloc a buffer the size of the file
  fread(buf, 1, buf_size, fd);        // and here it comes
  
  memcpy(&header, buf, sizeof(bsp_header));
  
  buf_pos = header.lump[1].offset; 
  plane = new bsp_plane[header.lump[1].length / sizeof(bsp_plane)];
  memcpy(plane, buf + buf_pos, header.lump[1].length);
 
  buf_pos = header.lump[2].offset;
  vertex = new vertex3f_t[header.lump[2].length / sizeof(vertex3f_t)];
  memcpy(vertex, buf + buf_pos, header.lump[2].length);
  
	for(int i = 0; i < header.lump[2].length / sizeof(vertex3f_t); i++) {
		cout << vertex[i].x << ", " << vertex[i].y << ", " << vertex[i].z << endl;
	}
  buf_pos = header.lump[4].offset;
  node = new bsp_node[header.lump[4].length / sizeof(bsp_node)];
  memcpy(node, buf + buf_pos, header.lump[4].length);

  buf_pos = header.lump[5].offset;
  texinfo.resize(header.lump[5].length / sizeof(bsp_texinfo));
  for(int i = 0; i < (header.lump[5].length / sizeof(bsp_texinfo)); i++) {
    texinfo[i].first = new bsp_texinfo;
    memcpy(texinfo[i].first, buf + buf_pos + i * sizeof(bsp_texinfo),
	   sizeof(bsp_texinfo));
    texinfo[i].second = new Texture(texinfo[i].first->texture_name);
  }
  
  buf_pos = header.lump[6].offset;
  face = new bsp_face[header.lump[6].length / sizeof(bsp_face)];
  memcpy(face, buf + buf_pos, header.lump[6].length);
  
  buf_pos = header.lump[7].offset;
  lightmap_data = new Uint8[header.lump[7].length];
  memcpy(lightmap_data, buf + buf_pos, header.lump[7].length);
  
  buf_pos = header.lump[8].offset;
  leaf = new bsp_leaf[header.lump[8].length / sizeof(bsp_leaf)];
  memcpy(leaf, buf + buf_pos, header.lump[8].length);
  
  buf_pos = header.lump[9].offset;
  leaf_face = new Uint16[header.lump[9].length / sizeof(Uint16)];
  memcpy(leaf_face, buf + buf_pos, header.lump[9].length);

  buf_pos = header.lump[11].offset;
  edge = new Uint16[header.lump[11].length / 2];
  memcpy(edge, buf + buf_pos, header.lump[11].length);

  buf_pos = header.lump[12].offset;
  face_edge = new Uint32[header.lump[12].length / sizeof(Uint32)];
  memcpy(face_edge, buf + buf_pos, header.lump[12].length);

	// Generate the display list now
	GLfloat texx, texy;
	dlist = glGenLists(1);
	glNewList(dlist, GL_COMPILE);
		glEnable(GL_TEXTURE_2D);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

		for(int f = 0; f < header.lump[6].length / sizeof(bsp_face); f++) {
			int first_edge = face[f].first_edge;

			glBindTexture(GL_TEXTURE_2D, texinfo[face[f].texture_info].second->tex);
			glBegin(GL_POLYGON); 
			for(int e = 0; e < face[f].num_edges; e++) {
				int tmp = face_edge[e + first_edge];
	
			/* Assign proper values to a and b: if tmp is positive a comes first,
		   	otherwise b comes first; */
				int a, b;
				if(tmp >= 0) {
				a = tmp * 2;
				b = tmp * 2 + 1;
			} else {
				a = -tmp * 2 + 1;
				b = -tmp * 2;
			}
	
			if(e == 0) {            // draw only vertex b if it isn't the first edge	
				texx = vertex[edge[a]].x * texinfo[face[f].texture_info].first->u_axis.x +
						 vertex[edge[a]].y * texinfo[face[f].texture_info].first->u_axis.y +
						 vertex[edge[a]].z * texinfo[face[f].texture_info].first->u_axis.z +
						 texinfo[face[f].texture_info].first->u_offset;
				texx /= texinfo[face[f].texture_info].second->w;
				texy = vertex[edge[a]].x * texinfo[face[f].texture_info].first->v_axis.x +
						 vertex[edge[a]].y * texinfo[face[f].texture_info].first->v_axis.y + 
						 vertex[edge[a]].z * texinfo[face[f].texture_info].first->v_axis.z + 
						 texinfo[face[f].texture_info].first->v_offset;
				texy /= texinfo[face[f].texture_info].second->h;
				glTexCoord2f(texx, texy);
				glVertex3fv(&vertex[edge[a]].x);
			}
			texx = vertex[edge[b]].x * texinfo[face[f].texture_info].first->u_axis.x +
					 vertex[edge[b]].y * texinfo[face[f].texture_info].first->u_axis.y +
					 vertex[edge[b]].z * texinfo[face[f].texture_info].first->u_axis.z +
					 texinfo[face[f].texture_info].first->u_offset;
			texx /= texinfo[face[f].texture_info].second->w; 
		
			texy = vertex[edge[b]].x * texinfo[face[f].texture_info].first->v_axis.x +
					 vertex[edge[b]].y * texinfo[face[f].texture_info].first->v_axis.y +
					 vertex[edge[b]].z * texinfo[face[f].texture_info].first->v_axis.z +
  	   	     texinfo[face[f].texture_info].first->v_offset;
			texy /= texinfo[face[f].texture_info].second->h;
			glTexCoord2f(texx, texy);
			glVertex3fv(&vertex[edge[b]].x);
  			}  
 	 	 	glEnd();
		}
		glDisable(GL_TEXTURE_2D);
  	glEndList();
	return 0;
}


void Map::render() {
  GLenum errcode;
  const GLubyte *errstring;
  
  
  glPushMatrix();
  glCallList(dlist);
  glPopMatrix();

  if((errcode = glGetError()) != GL_NO_ERROR) {
    cerr << "OGL Error: " << gluErrorString(errcode) << '\n';
  } 
}
