/*
    Proteus RPG Engine - A flexible 3D story based RPG engine.
    Copyright (C) 2002  Sam Yatchmenoff

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __model_h_
#define __model_h_

#include <sstream>
#include <string>
#include <GL/gl.h>
#include "visual.h"

using namespace std;

struct Vertex {
	float x, y, z;
	int bone;
};

struct Triangle {
	int a, b, c;
};

struct Mesh {
	int num_triangles;
	int *tris;
};

struct Node {
	float x, y, z;
	int num_children;
	Node **children;
	int mesh;
};

class Model : public PRVisual {
	Vertex loc;
	Vertex rot;

	int num_vertices;
	Vertex *vertices;

	int num_triangles;
	Triangle *triangles;

	int num_meshes;
	Mesh *meshes;

	Node *root;          // root node of the skeleton

	GLfloat r, g, b;
public:
	int selected;

	Model();
	~Model();
	void set_location(float x, float y, float z);
	void set_rotation(float x, float y, float z);
	void color(float, float, float);
	void render();
	void render_mesh(int);
	void render_skeleton(Node *);
	int load(const string &);
	void load_skeleton(istringstream &, Node *);
};

#endif

