/*
    Proteus RPG Engine - A flexible 3D story based RPG engine.
    Copyright (C) 2002  Sam Yatchmenoff

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __object_h_
#define __object_h_

#include <string>
#include <map>
#include <list>
#include "resman.h"
#include "model.h"
#include "physicalobject.h"
#include "character.h"

using namespace std;

struct Effect {
	string type;  			// modify ( may be only one )
	string subtype;  		// modify: add, subtract, set
	string targettype;	// stat, attribute, etc.
	string targetname;	// HP, MP, Strength, Dexterity, etc.
	string value;        // the source value, often an int
};

struct Action {
	string type;			// use, equip, etc.
	string target;			// possible targets, ex: "character", another type name
	list<Effect> effects;
};
	
class Object : public PhysicalObject {
 public: 
  Object();
	//Object(const Object &);
	int load(const string &filename);
	string get_type() { return type; };
	int get_attribute(const string &str) { return attributes[str]; }
 	Resource<Model> get_model() { return model; };
 
	void use_on(Character *target);
 private:
	string type;
	Resource<Model> model;
	list<Action> actions;
	map<string, int> attributes;  // name and value
};

ostream &operator<<(ostream &s, Object &o);

#endif
