/*
    Proteus RPG Engine - A flexible 3D story based RPG engine.
    Copyright (C) 2002  Sam Yatchmenoff

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <windows.h>

#include <iostream>
#include "party.h"
#include <GL/gl.h>
#include "model.h"
#include "types.h"
#include "visuals.h"
#include "milkshape.h"
#include "tinyxml.h"
#include "main.h"

using namespace std;

Party::Party() {
	location.x = 0; location.y = 0; location.z = 0;
}

void Party::update() {
}

void Party::draw_adventure() {
  Model *visual;
  visual = characters[0]->get_model();
  visual->set_location(location.x, location.y, location.z);
  visual->set_rotation(0, yaw, 0);
  add_visual(visual);
}

void Party::draw_battle() {
}

int Party::load() {
#ifdef DEBUG
  cout << "Loading party\n";
#endif
  
	string dirname = get_module_name() + "/party";

  string partycfg = dirname + "/party.xml";
	TiXmlDocument doc(partycfg);
  if(!doc.LoadFile()) {
    cout << "Error loading: " << partycfg << '\n';
    return -1;
  }
  TiXmlNode *node;
	TiXmlNode *xyz;

	assert(node = doc.FirstChild("Party"));

	assert(node = node->FirstChild("Location"));
	assert(node = node->FirstChild("X"));
	location.x = atoi(node->FirstChild()->ToText()->Value().c_str());
	assert(node = node->NextSibling("Y"));
	location.y = atoi(node->FirstChild()->ToText()->Value().c_str());
  assert(node = node->NextSibling("Z"));
	location.z = atoi(node->FirstChild()->ToText()->Value().c_str());

	assert(node = doc.FirstChild("Party")->FirstChild("CharacterList"));

  for(node = node->FirstChild("Entry"); node; node = node->NextSibling()) {
    characters.push_back(Resource<Character>(node->FirstChild()->ToText()->Value(), Level));
  }
  return 0;
}

Character *Party::get_character(int which) {
  return characters[which];
}

void Party::add_obj_to_inv(Resource<Object> obj) {
	inv.push_back(obj);
}

list<Resource<Object> > &Party::get_inv() {
	return inv;
}
