/*
    Proteus RPG Engine - A flexible 3D story based RPG engine.
    Copyright (C) 2002  Sam Yatchmenoff

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __resman_h
#define __resman_h

#include <string>
#include <map>
#include <vector>

#define RESERVED_IDS 1

using namespace std;

typedef unsigned long ResID;

enum ResCacheType { Temp, Level, Game };

struct ResEntry {
  string filename;
  ResCacheType cache_type;
  ResID id;
  void *data;
};

struct _resman_stuff {
	vector<ResEntry> entries;
	map<string,ResID> by_filename; // map[n].second are indices into entries
	int ID_count;
};

int init_resman(void);
ResID resman_register(const string &filename, ResCacheType cachetype);
ResID resman_find_id_by_filename(const string &);
ResID resman_add_entry(const string &, ResCacheType);
_resman_stuff &resman_get_stuff();

template<class T> T *resman_ptr_for_id(ResID id) {
	//cout << "resman_ptr_for_id(" << id << ")\n";
	if(id > resman_get_stuff().ID_count) {   // bad id
	cout << "woops!\n";
		return NULL;    
	} else {
		ResEntry *entry = &resman_get_stuff().entries[id - 1];
		if(entry->data == NULL) {
			entry->data = new T;
			((T*)(entry->data))->load(entry->filename);
		}
		return (T*)entry->data;
	}
}

// Resource, almost an interface to resman
template<class T> class Resource {
 public:
  Resource() { id = 0; };
	Resource(const Resource &res) : id(res.id) {};
	Resource(const string &name, ResCacheType cache_type) { id = resman_register(name, cache_type); };
  ~Resource() {};
  T& operator*() { return *resman_ptr_for_id<T>(id); };
  T *operator->() { return &(operator*()); };
  operator T*() { return &(operator*()); };
 private:
  ResID id;     // id for use with resman
};

#endif
