/*
    Proteus RPG Engine - A flexible 3D story based RPG engine.
    Copyright (C) 2002  Sam Yatchmenoff

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <windows.h>

#include <GL/gl.h>
#include <GL/glu.h>
#include <SDL_image.h>
#include "texture.h"
#include <iostream>
#include <string>
#include "main.h"

using namespace std;

Texture::Texture(const string &texname, bool mipmap) {
	string fullname = get_module_name() + "/data/textures/" + texname + ".png";
#ifdef DEBUG
	cout << "[TEX] loading: " << fullname << '\t';
#endif
	SDL_Surface *surface;
	glGenTextures(1, &tex);
	if((surface = IMG_Load(fullname.c_str())) == NULL) {
#ifdef DEBUG
		cout << "\t[FAILED]\n";
#endif
		return;
	} else {
		w = surface->w;
		h = surface->h;
		glBindTexture(GL_TEXTURE_2D, tex);
		if(mipmap) {
			gluBuild2DMipmaps(GL_TEXTURE_2D, 3, surface->w, surface->h, GL_RGBA, GL_UNSIGNED_BYTE, surface->pixels);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		} else {
			glTexImage2D(GL_TEXTURE_2D, 0, 3, surface->w, surface->h, 0, GL_RGBA, GL_UNSIGNED_BYTE, surface->pixels);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		}
		SDL_FreeSurface(surface);
	}
#ifdef DEBUG
	cout << "\t[OK]\n";
#endif
}

int Texture::load(const string &texname) {
	string fullname = get_module_name() + "/data/textures/" + texname + ".png";
	cout << "[TEX] loading: " << fullname << '\t';
	SDL_Surface *surface;
	glGenTextures(1, &tex);
	if((surface = IMG_Load(fullname.c_str())) == NULL) {
		cout << "\t[FAILED]\n";
		return -1;
	}
	w = surface->w;
	h = surface->h;
	glBindTexture(GL_TEXTURE_2D, tex);
	gluBuild2DMipmaps(GL_TEXTURE_2D, 3, surface->w, surface->h, GL_RGBA, GL_UNSIGNED_BYTE, surface->pixels);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	SDL_FreeSurface(surface);
	cout << "\t[OK]\n";
	return 0;
}																																  
