/*
    Proteus RPG Engine - A flexible 3D story based RPG engine.
    Copyright (C) 2002  Sam Yatchmenoff

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <windows.h>
#include <SDL.h>
#include <GL/gl.h>
#include <iostream>
#include "universe.h"
#include "resman.h"
#include "map.h"
#include "input.h"
#include "visuals.h"
#include "tinyxml.h"
#include "text.h"
#include "video.h"

using namespace std;

Universe::Universe(GameOptions *options) {
	blarg = false;

#ifdef DEBUG
	cout << "Loading module " << options->module_name << '\n';
#endif

	TiXmlDocument doc;
	TiXmlNode *node;
	TiXmlText *text;
  
  if(!doc.LoadFile(options->module_name + "/module.xml")) {
    cerr << "Error: " << options->module_name << " is not a valid module\n";
    blarg = true;
    return;
  }
  node = doc.FirstChild("Module");
  node = node->FirstChild("Name");
  text = node->FirstChild()->ToText();
  module_name = text->Value();
#ifdef DEBUG
  cout << "Module Name: " << text->Value() << '\n';
#endif
 
  if(!doc.LoadFile(options->module_name + "/scenes/scenes.xml")) {
    cerr << "Error: Can't find scene list\n";
    blarg = true;
    return;
  }
  
  for(node = doc.FirstChild("SceneList")->FirstChild("Entry"); node;
      node = node->NextSibling("Entry")) {
    text = node->FirstChild()->ToText();
    scenes.push_back(Resource<Scene>(text->Value(), Level));
  }

  if(party.load() == -1) {
    blarg = true;
    return;
  }
}

bool Universe::error() {
  return blarg;
}

int Universe::big_bang() { 

  cout << "...and so it begins..." << endl;
  // should now check out options
  
  adventure_init();
  battle_init();
  menu_init();

  adventure_start();
  
  game_loop();
  return 0;
}

void Universe::game_loop() {
/* for fps counter */
	GLint T0 = 0;
	GLint frames = 0;
	GLint t;
	GLfloat seconds, fps;
/* for fps counter */

	while(game_state != GS_QUIT) {
		input_update(); // should this be in another thread?
		switch(game_state) {
		case GS_ADVENTURE:
			adventure_update();    
			break;
		case GS_BATTLE:
			battle_update();
			break;
		case GS_MENU:
			menu_update();
			break;
		}
		glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
		add_visual(new Text(0, 0, 1.0, 1.0, 1.0, "FPS: %f", fps));
		render_visuals();
		SDL_GL_SwapBuffers();
		if(input_get_key(I_KEY_S) == true) {
			input_set_key(I_KEY_S, false);
			video_take_screenshot();
		}
	
		frames++;
		t = SDL_GetTicks();
		if (t - T0 >= 1000) {
			seconds = (t-T0) / 1000.0;
			fps = frames / seconds;
			T0=t;
			frames=0;
		}
	}
}

void Universe::set_game_state(int mode) {
	game_state = mode;
}

void Universe::change_scene(int which) {
	// TODO: Have resman free some memory!
	party.set_cur_scene(which);
	current_scene = which;
}

